"use strict";

//angmoba.controller("GameCtrl", ["$scope", function($scope) {
angmoba.controller("GameCtrl", ["$scope","RTC","AmobaGame", function($scope,RTC,Game) {

	//Loading factories dynamically
	//var $injector = angular.injector(["ang-moba"]);
	//var RTC = $injector.get("RTC");
	//var Game = $injector.get("AmobaGame");
	
	//Closures
	var board = null;
	var initiator = false;
	var myTurn = true;
	var winner = "";
	var hash = location.hash.toString().replace(/[^\w-]/g, "");

	//Bindings
	$scope.isTestMode = (hash == "testmode");

	$scope.rows = [];

	$scope.isDataChannelSupported = !!window.chrome;

	$scope.channelName = ($scope.isTestMode || $scope.isDataChannelSupported ? hash : "");

	$scope.message = $scope.isDataChannelSupported ? "Welcome! You may initiate a new game now." : "chrome required";
	if ($scope.isTestMode)
		$scope.message = "Test mode";
	
	$scope.leaveMode = function() {
		return $scope.rows.length || $scope.channelName;
	}

	$scope.createButtonText = function() {
		return $scope.leaveMode() ? "leave game " + $scope.channelName : "new game";
	}

	$scope.createButtonClass = function() {
		return $scope.leaveMode() ? "button cancel-button" : "button create-button";
	}

	$scope.boardEnabled = function() {
		if (winner)
			return false;
		if ($scope.isTestMode)
			return true;
		return RTC.connected() && myTurn;
	};

	$scope.gamer = function() {
		return board && board.gamer ? board.gamer : "";
	}
	
	//Button handlers
	$scope.createChannel = function() {
		//board = Game.createBoard(initiator? "x" : "o");
		//$scope.rows = board.rows;
		//return;
		
		if ($scope.isTestMode)
			return;
		
		if ($scope.leaveMode()) {
			location.href = "";
		} else {
			$scope.channelName = location.hash = RTC.open();			
			initiator = true;
			$scope.message = "Created game " + $scope.channelName + ". Send the actual URL to your partner. Waiting for the partner to connect...";
		}

	}




	//Event handlers - broadcasted by RTC
	
	$scope.$on("rtc_onopen", function(){
		//2 peers connected
		//console.info("rtc_onopen called", userId);
		$scope.rows = [];
		myTurn = true;
		$scope.message = $scope.isTestMode ? "Test mode is active" : "Connection established with the remote user. Let's play!";
		
		board = Game.createBoard(initiator? "x" : "o");
		$scope.rows = board.rows;
		if (!$scope.isTestMode)
			$scope.$digest();
	});

	$scope.step = function(cell) {
		if (!$scope.boardEnabled() || cell.gamer)
			return;

		winner = board.step($scope.gamer(), cell.r,cell.c);
		$scope.message = winner ? "You won!" : "Step taken. Waiting for the remote user to step...";
		myTurn = false;
		if ($scope.isTestMode) {
			myTurn = true;
			board.gamer = (board.gamer=="o" ? "x" : "o");
		} else
				RTC.send(cell.toJson());
	};

	$scope.$on("rtc_onstep", function(event, message) {
		var step = JSON.parse(message);
		winner = board.step(step.gamer, step.r, step.c);

		$scope.message = winner ? "You've lost!" : "The remote user took a step. Your turn!";
		myTurn = true;
		if (!$scope.isTestMode)
			$scope.$digest();
	});

	$scope.$on("rtc_onleave", function(){
		$scope.message = "The remote user has left the game.";
		$scope.channelName = "";
		location.hash = "";
		if (!$scope.isTestMode)
			$scope.$digest();
	});
	
	$scope.init = function() {
		if ($scope.channelName) {
			$scope.message = "Joining to game " + $scope.channelName + "...";
			initiator = false;
			if ($scope.isTestMode) {
				$scope.$broadcast("rtc_onopen", 0);
			} else {
				RTC.connect($scope.channelName);				
			}
		}
	}

}]);
